﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Persistence;

namespace doLogic
{
    public class LocationManagers : PersistentList<LocationManager>
    {
        public Location Location { get; protected set; }

        public static LocationManagers For(int locationID)
        {
            using (Connection cn = Database.Open())
            {
                LocationManagers list = new LocationManagers();
                list.Load("ForLocation", new { LocationID = locationID }, cn);

                list.Location = Location.Load(locationID, cn);

                foreach (LocationManager lm in list)
                {
                    lm.Location = list.Location;
                    lm.LoadManager(cn);
                }
                return list;
            }
        }

        private LocationManagers() { }

        public LocationManagers Matching(string searchCriteria)
        {
            if (searchCriteria.IsEmpty())
                return this;

            searchCriteria = searchCriteria.ToLower();

            LocationManagers list = new LocationManagers();

            foreach (LocationManager lm in this)
                if (lm.Manager.Equals(searchCriteria))
                    list.Add(lm);
            return list;
        }
    }

    [Persistable("Manager_Locations", "ID", false)]
    public class LocationManager
    {
        public LocationManager() { }

        internal LocationManager(Location location, Manager manager, bool isPrimary)
        {
            this.Manager = manager;
            this.ManagerID = manager.ManagerID;
            this.Location = location;
            this.LocationID = location.LocID;
            this.IsPrimary = isPrimary;
        }

        public Manager Manager { get; internal set; }
        public Location Location { get; internal set; }

        internal void LoadManager(Connection cn)
        {
            this.Manager = Manager.Load(this.ManagerID, cn);
        }

        internal static LocationManager Store(int locationID, int managerID, bool isPrimary)
        {
            LocationManager lm = new LocationManager();
            lm.ManagerID = managerID;
            lm.LocationID = locationID;
            lm.IsPrimary = isPrimary;
            Database.Store(lm);
            return lm;
        }

        internal static void Delete(int locationID, int managerID)
        {
            LocationManager lm = new LocationManager();
            lm.ManagerID = managerID;
            lm.LocationID = locationID;
            Database.Delete(lm);
        }

        #region Manager fields

        public string FullName { get { return this.Manager.FullName; } }

        public string FirstName
        {
            get { return this.Manager.FirstName; }
            set { this.Manager.FirstName = value; }
        }

        public string LastName
        {
            get { return this.Manager.LastName; }
            set { this.Manager.LastName = value; }
        }

        public string MiddleName
        {
            get { return this.Manager.MiddleName; }
            set { this.Manager.MiddleName = value; }
        }

        public string Title
        {
            get { return this.Manager.Title; }
            set { this.Manager.Title = value; }
        }

        public string Phone
        {
            get { return this.Manager.Phone; }
            set { this.Manager.Phone = value.ToNumericOnly(); }
        }

        public string PhoneFormatted
        {
            get { return this.Manager.PhoneFormatted; }
            set { this.Manager.PhoneFormatted = value; }
        }

        public string Extension
        {
            get { return this.Manager.Extension; }
            set { this.Manager.Extension = value; }
        }

        public string Email
        {
            get { return this.Manager.Email; }
            set { this.Manager.Email = value; }
        }
        #endregion

        #region Location fields

        public string LocationName
        {
            get { return this.Location.LocationName; }
            set { this.Location.LocationName = value; }
        }

        public int DivisionID
        {
            get { return this.Location.DivID; }
            set { this.Location.DivID = value; }
        }

        public string DivisionName
        {
            get { return this.Location.DivisionName; }
            set { this.Location.DivisionName = value; }
        }
        
        #endregion

        #region Manager_Location persistable properties

        public int ID { get; set; }
        public int ManagerID { get; set; }
        public int LocationID { get; set; }
        public bool IsPrimary { get; set; }
        
        #endregion

        public string IsPrimaryDisplay
        {
            get { return IsPrimary.ToYesNo(); }
            set { IsPrimary = value.ToBoolean(); }
        }
    }
}
