﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Persistence;

namespace doLogic
{
    public class Managers : PersistentList<Manager>
    {
        public static Managers For(Location location)
        {
            return Managers.ForLocation(location.ClientCode, location.LocID);
        }

        public static Managers For(Division division)
        {
            return Managers.ForDivision(division.ClientCode, division.DivID);
        }

        public static Managers For(Client client)
        {
            return Managers.For(client.ClientCode);
        }

        public static Managers For(int clientCode)
        {
            return new Managers(clientCode);
        }

        public static Managers ForLocation(int clientCode, int locationID)
        {
            Managers list = new Managers();
            list.ClientCode = clientCode;
            list.LocationID = locationID;
            list.Load("ForLocation");
            return list;
        }

        public static Managers ForDivision(int clientCode, int divisionID)
        {
            Managers list = new Managers();
            list.ClientCode = clientCode;
            list.DivisionID = divisionID;
            list.Load("ForDivision");
            return list;
        }

        public Managers Matching(string searchCriteria)
        {
            if (searchCriteria.IsEmpty())
                return this;

            Managers list = new Managers();

            foreach (Manager m in this)
                if (m.Equals(searchCriteria))
                    list.Add(m);
            return list;
        }

        internal Managers() { }

        private Managers(int clientCode)
        {
            this.ClientCode = clientCode;
            this.Load("ForClient");
        }

        public void Delete(int managerID)
        {
            Manager m = this.ItemById(managerID);
            if (m != null)
                this.Remove(m);

            m = new Manager();
            m.ManagerID = managerID;
            Database.Delete(m);
        }

        public int ClientCode { get; set; }
        public int LocationID { get; set; }
        public int DivisionID { get; set; }
    }

    [Persistable("Managers", "ManagerID", false)]
    public class Manager
    {
        public static void AssignToDivision(int divisionID, int managerID, bool isPrimary)
        {
            DivisionManager.Store(divisionID, managerID, isPrimary);
        }

        public void AssignToDivision(int divisionID, bool isPrimary)
        {
            if (this.ManagerID == 0)
                this.Store();
            DivisionManager.Store(divisionID, this.ManagerID, isPrimary);
        }

        public static void UnassignFromDivision(int divisionID, int managerID)
        {
            DivisionManager.Delete(divisionID, managerID);
        }

        public void UnassignFromDivision(int divisionID)
        {
            DivisionManager.Delete(divisionID, this.ManagerID);
        }

        public static void AssignToLocation(int locationID, int managerID, bool isPrimary)
        {
            LocationManager.Store(locationID, managerID, isPrimary);
        }

        public void AssignToLocation(int locationID, bool isPrimary)
        {
            if (this.ManagerID == 0)
                this.Store();
            LocationManager.Store(locationID, this.ManagerID, isPrimary);
        }

        public static void UnassignFromLocation(int locationID, int managerID)
        {
            LocationManager.Delete(locationID, managerID);
        }

        public void UnassignFromLocation(int locationID)
        {
            LocationManager.Delete(locationID, this.ManagerID);
        }

        public bool Equals(string searchCriteria)
        {
            if (this.FirstName.StartsWith(searchCriteria, StringComparison.OrdinalIgnoreCase))
                return true;
            else if (this.LastName.StartsWith(searchCriteria, StringComparison.OrdinalIgnoreCase))
                return true;
            else if (this.Title.ToLower().Contains(searchCriteria.ToLower()))
                return true;
            else if (this.Phone.Contains(searchCriteria))
                return true;
            else if (this.Email.ToLower().Contains(searchCriteria.ToLower()))
                return true;

            return false;
        }

        public static Manager Load(int managerID, Connection cn)
        {
            return new Manager(managerID, cn);
        }

        //constructors
        public Manager() { }

        public Manager(int managerID)
        {
            ManagerID = managerID;
            Database.Load(this);
        }

        private Manager(int managerID, Connection cn)
        {
            this.ManagerID = managerID;
            Database.Load(this, cn);
        }

        public string PhoneFormatted
        {
            get
            {
                return Phone.ToPhoneFormat();
            }
            set
            {
                Phone = value.ToNumbersOnly().TruncateTo(10);
            }
        }

        public string FullName
        {
            get
            {
                return String.Format("{0} {1}", FirstName, MiddleName).Trim() + " " + LastName;
            }
        }

        //persistent properties
        public int ManagerID { get; set; }
        public int ClientCode { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string MiddleName { get; set; }
        public string Title { get; set; }
        private string _phone = "";
        public string Phone { get { return _phone; } set { _phone = value.ToNumbersOnly(); } }
        public string Extension { get; set; }
        public string Email { get; set; }

        public int Store() { return Database.Store(this); }
        //public void Load() { Database.Load(this); }
        //public bool Delete() { return Database.Delete(this); }
    }
}
